import { world, ItemStack, system, BlockPermutation } from "@minecraft/server";
import { ActionFormData } from "@minecraft/server-ui";

var namespace = "rzb_dcocrfnt";
var rl = (path) => {
    return `${namespace}:${path}`;
};
var propertyName = rl("save_data");

function getSaveData(player) {
    let props = player.getDynamicProperty(propertyName);
    if (typeof props === "string") {
        return getSaveDataWithDefaults(JSON.parse(props));
    }
    return { givenBook: false, viewedPages: [] };
}
function setSaveData(player, data) {
    player.setDynamicProperty(propertyName, JSON.stringify(data));
}
function getSaveDataWithDefaults(data) {
    return {
        givenBook: data.givenBook,
        viewedPages: data.viewedPages ?? [],
    };
}

var Book = class {
    pages = {};
    trackPages;
    constructor(index, trackPages = true) {
        this.trackPages = trackPages;
        this.addPage("index", index);
    }
    addPage(name, page) {
        this.pages[name] = page;
    }
    async openBook(params) {
        await this.openPage("index", params);
    }
    async openPage(page, params) {
        if (this.trackPages) {
            let saveData = getSaveData(params.player);
            if (!saveData.viewedPages.includes(page)) {
                saveData.viewedPages.push(page);
                setSaveData(params.player, saveData);
            }
        }
        params.currentPage = page;
        await this.pages[page].show({
            book: this,
            player: params.player,
            ...params,
        });
    }
};

var Page = class {
    type;
    _title;
    constructor({ type, title }) {
        this.type = type;
        this._title = this.state(title);
    }
    title(params) {
        return this._title(params);
    }
    state(thing) {
        return typeof thing === "function" ? thing : () => thing;
    }
};
var ActionPage = class extends Page {
    _body;
    _buttons;
    constructor({ title, body, buttons }) {
        super({ type: "action", title });
        this._body = this.state(body);
        this._buttons = this.state(buttons);
    }
    async show(params) {
        const form = new ActionFormData()
            .title(this.title(params))
            .body(this.body(params));
        const hydratedButtons = this.buttons(params);
        hydratedButtons.forEach((value) => value.attach(form));
        const response = await form.show(params.player);
        if (!response.canceled && response.selection !== void 0) {
            const element = hydratedButtons[response.selection];
            if (element) {
                await element.submit(params);
            }
        }
    }
    body(params) {
        return this._body(params);
    }
    buttons(params) {
        return this._buttons(params);
    }
};
var handleAction = async (params, action) => {
    if (action) {
        const subAction = await action(params);
        if (subAction) {
            await handleAction(params, subAction);
        }
    }
};

var Button = class {
    text;
    action;
    constructor({ text, action }) {
        this.text = text;
        this.action = action;
    }
    attach(page) {
        if (page instanceof ActionFormData) {
            page.button(this.text);
        }
    }
    async submit(params) {
        if (this.action) {
            await handleAction(params, this.action);
        }
    }
};
var IconButton = class {
    text;
    icon;
    action;
    constructor({ text, icon, action }) {
        this.text = text;
        this.action = action;
        this.icon = icon;
    }
    attach(page) {
        if (page instanceof ActionFormData) {
            page.button(this.text, this.icon);
        }
    }
    async submit(params) {
        if (this.action) {
            await handleAction(params, this.action);
        }
    }
};
var bookItemId = rl("guide_book");                          // --> Esta variable NO se ha utilizado
var nextButtonKey = `guidebook.${namespace}.button.next`;   // --> Esta variable NO se ha utilizado
var backButtonKey = `guidebook.${namespace}.button.back`;
var firstPage = "index";
var initGuideBook = () => {
    const guideBook = createGuideBook();

    world.afterEvents.itemUse.subscribe(async ({ itemStack, source }) => {
        if (itemStack.typeId === "rzb_dcocrfnt:guide_book") {                        
            await guideBook.openBook({ player: source });
        }
    });

    world.afterEvents.playerSpawn.subscribe(async (event) => {
        let saveData = getSaveData(event.player);
        if (!saveData.givenBook) {
            event.player.dimension.spawnItem(
                new ItemStack(rl("guide_book"), 1),
                event.player.location
            );
            setSaveData(event.player, { ...saveData, givenBook: true });
        }
    });
};

function createGuideBook() {
    const index = new LinkablePage(firstPage);
    const tutorial = new LinkablePage("tutorial", index);                   // OJO! El primer argumento (en este caso "tutorial") sirve para decidir 2 cosas:    1. Qué icono se mostrará junto al botón.    2. Qué textos del en_US.lang serán mostrados
    const tutobiomat = new LinkablePage("tuto_biomat", tutorial);        
    const tutosproutingtable = new LinkablePage("tuto_sproutingtable", tutorial);
    const tutosproutingtableFuel = new CraftingPage("tuto_sproutingtable", tutosproutingtable);
/*    const sproutingtableFuel = new LinkablePage("sproutingtable_fuel", tutosproutingtable);               BORRADO
    const sproutingtableInput = new LinkablePage("sproutingtable_input", tutosproutingtable);
    const sproutingtableOutput = new LinkablePage("sproutingtable_output", tutosproutingtable);     */
    const tutonaturebench = new LinkablePage("tuto_naturebench", tutorial);
    const naturebenchRecipe = new CraftingPage("tuto_naturebench", tutonaturebench);
/*    const naturebenchInput = new LinkablePage("naturebench_input", tutonaturebench);               BORRADO
    const naturebenchOutput = new LinkablePage("naturebench_output", tutonaturebench);
    const naturebenchDecorations = new LinkablePage("naturebench_decorations", tutonaturebench);        */
    const tutoDecobrush = new LinkablePage("tuto_decobrush", tutorial);
    const tutoDecobrushRecipe = new CraftingPage("tuto_decobrush", tutoDecobrush);
    const tutoWarning = new LinkablePage("tuto_warning", tutorial);
    const credits = new LinkablePage("credits", index);

    const book = new Book(
        index.actionPage((ctx) => [
            tutorial.pageLinkButton(ctx),
            credits.pageLinkButton(ctx),
        ]),
        true
    );
    tutorial.addActionPage(book, (ctx) => {
        let buttons = [
            tutobiomat.pageLinkButton(ctx),
            tutosproutingtable.pageLinkButton(ctx),
            tutonaturebench.pageLinkButton(ctx),
            tutoDecobrush.pageLinkButton(ctx),
            tutoWarning.pageLinkButton(ctx),
        ];
        buttons.push(tutorial.backButton());
        return buttons;
    });
    tutobiomat.addActionPage(book, (ctx) => [
        tutosproutingtable.nextPageButton(ctx),
        tutobiomat.backButton(),
    ]);
    tutosproutingtable.addActionPage(book, (ctx) => [
        tutosproutingtableFuel.pageLinkButton(ctx),
/*        sproutingtableFuel.pageLinkButton(ctx),               BORRADO
        sproutingtableInput.pageLinkButton(ctx),
        sproutingtableOutput.pageLinkButton(ctx),     */
        tutonaturebench.nextPageButton(ctx),
        tutosproutingtable.backButton(),
    ]);
    tutosproutingtableFuel.addActionPage(book);
/*    sproutingtableFuel.addActionPage(book);
    sproutingtableInput.addActionPage(book);
    sproutingtableOutput.addActionPage(book);       */
    tutonaturebench.addActionPage(book, (ctx) => [
        naturebenchRecipe.pageLinkButton(ctx),
/*        naturebenchInput.pageLinkButton(ctx),               BORRADO
        naturebenchOutput.pageLinkButton(ctx),
        naturebenchDecorations.pageLinkButton(ctx),     */
        tutoDecobrush.nextPageButton(ctx),
        tutonaturebench.backButton(),
    ]);
    naturebenchRecipe.addActionPage(book);
/*    naturebenchInput.addActionPage(book);               BORRADO
    naturebenchOutput.addActionPage(book);
    naturebenchDecorations.addActionPage(book);     */
    tutoDecobrush.addActionPage(book, (ctx) => [
        tutoDecobrushRecipe.pageLinkButton(ctx),
        tutoWarning.nextPageButton(ctx),
        tutoDecobrush.backButton(),
    ]);
    tutoDecobrushRecipe.addActionPage(book);
    tutoWarning.addActionPage(book, (ctx) => [
        tutoWarning.backButton(),
    ]);
    credits.addActionPage(book);
    return book;
}

var LinkablePage = class {
    parent;
    pageId;
    constructor(pageId, parent) {
        this.pageId = pageId;
        this.parent = parent;
    }
    localize(type) {
        return `guidebook.${namespace}.${this.pageId}.${type}`;
    }
    titleKey() {
        return this.localize("title");
    }
    bodyKey() {
        return this.localize("body");
    }
    backButton() {
        const btnText = this.parent
            ? {
                translate: `guidebook.${namespace}.button.back_page`,
                with: {
                    rawtext: [
                        { translate: this.parent ? this.parent.titleKey() : `error` },
                    ],
                },
            }
            : { translate: backButtonKey };
        return new Button({
            text: btnText,
            action: async (params) => {
                if (this.parent) {
                    params.book.openPage(this.parent.pageId, params);
                }
            },
        });
    }
    addActionPage(book, buttons = (ctx) => [this.backButton()]) {
        book.addPage(this.pageId, this.actionPage(buttons));
    }
    actionPage(buttons) {
        return new ActionPage({
            title: { translate: this.titleKey() },
            body: { translate: this.bodyKey(), with: ["\n", "\n\n"] },
            buttons,
        });
    }
    getIconPath(player, displayUnread = true) {
        return `textures/rb/dcn/guide_book/${this.pageId}${displayUnread && !hasPlayerReadPage(player, this.pageId) ? "_new" : ""
            }`;
    }
    nextPageButton(ctx, displayUnread = true) {
        return new IconButton({
            text: {
                translate: `guidebook.${namespace}.button.next`,
                with: { rawtext: [{ translate: this.titleKey() }] },
            },
            icon: this.getIconPath(ctx.player, displayUnread),
            action: async (params) => params.book.openPage(this.pageId, params),
        });
    }
    pageLinkButton(ctx, displayUnread = true) {
        return new IconButton({
            text: { translate: this.titleKey() },
            icon: this.getIconPath(ctx.player, displayUnread),
            action: async (params) => params.book.openPage(this.pageId, params),
        });
    }
};
var CraftingPage = class extends LinkablePage {
    constructor(pageId, parent, prefix = "craft") {
        super(`${prefix}_${pageId}`, parent);
    }
    titleKey() {
        return `guidebook.${namespace}.page.crafting`;
    }
};

function hasPlayerReadPage(player, pageId) {
    return getSaveData(player).viewedPages.includes(pageId);
}

initGuideBook();


export function getScore(target, obj) {
  try {
      const oB = world.scoreboard.getObjective(obj)
      return oB.getScore(typeof target == 'string' ? oB.getParticipants().find(pT => pT.displayName == target) : oB.getScore(target.scoreboard))
  } catch {
      return NaN
  }
};
export function setScore(target, obj, value) {
  try {
      const oB = world.scoreboard.getObjective(obj);
      oB.setScore(target, value);
      return getScore(target, obj);
  } catch {
      return NaN
  }
};
system.afterEvents.scriptEventReceive.subscribe(({ id, sourceEntity }) => {
    if (id === 'rzb_dcocrfnt:newslotsamount_seeds') {
        const amount = sourceEntity.getProperty("rzb_dcocrfnt:amount");
        const toraw = Math.floor(amount / 8);
        setScore("rzb_dcocrfnt:bio_materials_cr", "rzb_dcocrfnt:jig_computer.addon_stats", getScore("rzb_dcocrfnt:bio_materials_cr", "rzb_dcocrfnt:jig_computer.addon_stats") + toraw);
        for (let entity of world.getDimension("overworld").getEntities({ type: 'rzb_dcocrfnt:sprttable_slot_out', closest: 1 })) {
            entity.setProperty("rzb_dcocrfnt:amount", toraw);
            // Introducimos un pequeño retraso
            system.runTimeout(() => {
                const entityAmount = entity.getProperty("rzb_dcocrfnt:amount");                
                const slotin_decrease = entityAmount * 8;
                const slotin_newvalue = amount - slotin_decrease;
                sourceEntity.setProperty("rzb_dcocrfnt:amount", slotin_newvalue);
                setScore("rzb_dcocrfnt:dirt_sprouted_", "rzb_dcocrfnt:jig_computer.addon_stats", getScore("rzb_dcocrfnt:dirt_sprouted_", "rzb_dcocrfnt:jig_computer.addon_stats") + slotin_decrease);
            }, 1);
        }
    }
    if (id === 'rzb_dcocrfnt:newslotsamount_cobble') {
        const amount = sourceEntity.getProperty("rzb_dcocrfnt:amount");
        const toraw = Math.floor(amount / 4);
        setScore("rzb_dcocrfnt:bio_materials_cr", "rzb_dcocrfnt:jig_computer.addon_stats", getScore("rzb_dcocrfnt:bio_materials_cr", "rzb_dcocrfnt:jig_computer.addon_stats") + toraw);
        for (let entity of world.getDimension("overworld").getEntities({ type: 'rzb_dcocrfnt:sprttable_slot_out', closest: 1 })) {
            entity.setProperty("rzb_dcocrfnt:amount", toraw);
            system.runTimeout(() => {
                const entityAmount = entity.getProperty("rzb_dcocrfnt:amount");                
                const slotin_decrease = entityAmount * 4;
                const slotin_newvalue = amount - slotin_decrease;
                sourceEntity.setProperty("rzb_dcocrfnt:amount", slotin_newvalue);
                setScore("rzb_dcocrfnt:cobblestone_deco", "rzb_dcocrfnt:jig_computer.addon_stats", getScore("rzb_dcocrfnt:cobblestone_deco", "rzb_dcocrfnt:jig_computer.addon_stats") + slotin_decrease);
            }, 1);
        }
    }
    if (id === 'rzb_dcocrfnt:newslotsamount_wood') {
        const amount = sourceEntity.getProperty("rzb_dcocrfnt:amount");
        const toraw = Math.floor(amount / 2);
        setScore("rzb_dcocrfnt:bio_materials_cr", "rzb_dcocrfnt:jig_computer.addon_stats", getScore("rzb_dcocrfnt:bio_materials_cr", "rzb_dcocrfnt:jig_computer.addon_stats") + toraw);
        for (let entity of world.getDimension("overworld").getEntities({ type: 'rzb_dcocrfnt:sprttable_slot_out', closest: 1 })) {
            entity.setProperty("rzb_dcocrfnt:amount", toraw);
            system.runTimeout(() => {
                const entityAmount = entity.getProperty("rzb_dcocrfnt:amount");                
                const slotin_decrease = entityAmount * 2;
                const slotin_newvalue = amount - slotin_decrease;
                sourceEntity.setProperty("rzb_dcocrfnt:amount", slotin_newvalue);
                setScore("rzb_dcocrfnt:logs_sprouted_", "rzb_dcocrfnt:jig_computer.addon_stats", getScore("rzb_dcocrfnt:logs_sprouted_", "rzb_dcocrfnt:jig_computer.addon_stats") + slotin_decrease);
            }, 1);
        }
    }
});

function checkAndSetCooldown(player, cooldownDuration) {
    const tick = system.currentTick;
    const playerCooldown = player.cooldown1 ?? tick;
    if (playerCooldown - tick > 0) {
      return false;
    }
    player.cooldown1 = tick + cooldownDuration;
    return true;
}

const soundOptions1 = {
    volume: 1.0
};
const soundOptions2 = {
    volume: 0.5
};
const soundOptions3 = {
    volume: 0.85
};

// Clipping blocks RIGHT

world.beforeEvents.playerInteractWithBlock.subscribe((data) => {
    const {player, itemStack, block} = data;
    const blockState = block.permutation.getState("rzb_dcocrfnt:variation");
    const blockOrientation = block.permutation.getState("minecraft:cardinal_direction");

    if(itemStack?.typeId == "rzb_dcocrfnt:hedge_clipper") {
        
        // Vars blocks

        const blockVariations = {
            "rzb_dcocrfnt:deco_s11_p1_12_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s11_p1_16_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s11_p1_20_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s11_p1_24_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s2_p1_01_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s2_p1_03_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s2_p1_04_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s2_p1_06_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s2_p1_07_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s2_p1_09_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s2_p1_10_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s2_p1_12_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s2_p1_13_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s2_p1_15_vars": [0, 1, 2],
            "rzb_dcocrfnt:deco_s11_p1_14_vars": [0, 1, 2, 3],
            "rzb_dcocrfnt:deco_s11_p1_15_vars": [0, 1, 2, 3],
            "rzb_dcocrfnt:deco_s11_p1_18_vars": [0, 1, 2, 3],
            "rzb_dcocrfnt:deco_s11_p1_19_vars": [0, 1, 2, 3],
            "rzb_dcocrfnt:deco_s11_p1_22_vars": [0, 1, 2, 3],
            "rzb_dcocrfnt:deco_s11_p1_23_vars": [0, 1, 2, 3],
            "rzb_dcocrfnt:deco_s11_p1_26_vars": [0, 1, 2, 3],
            "rzb_dcocrfnt:deco_s11_p1_27_vars": [0, 1, 2, 3],
            "rzb_dcocrfnt:deco_s11_p1_08_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s11_p1_09_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s11_p1_10_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s11_p1_11_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s11_p1_13_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s11_p1_17_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s11_p1_21_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s11_p1_25_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s5_p1_13_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s5_p1_14_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s5_p1_15_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s5_p1_16_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s5_p1_17_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s5_p1_18_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s5_p1_19_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s5_p1_20_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s5_p1_21_vars": [0, 1, 2, 3, 4],
            "rzb_dcocrfnt:deco_s2_p1_26_vars": [0, 1, 2, 3, 4, 5],
            "rzb_dcocrfnt:deco_s2_p2_02_vars": [0, 1, 2, 3, 4, 5],
            "rzb_dcocrfnt:deco_s2_p2_06_vars": [0, 1, 2, 3, 4, 5],
            "rzb_dcocrfnt:deco_s2_p2_10_vars": [0, 1, 2, 3, 4, 5],
            "rzb_dcocrfnt:deco_s2_p2_14_vars": [0, 1, 2, 3, 4, 5],
            "rzb_dcocrfnt:deco_s2_p1_02_vars": [0, 1, 2, 3, 4, 5, 6],
            "rzb_dcocrfnt:deco_s2_p1_05_vars": [0, 1, 2, 3, 4, 5, 6],
            "rzb_dcocrfnt:deco_s2_p1_08_vars": [0, 1, 2, 3, 4, 5, 6],
            "rzb_dcocrfnt:deco_s2_p1_11_vars": [0, 1, 2, 3, 4, 5, 6],
            "rzb_dcocrfnt:deco_s2_p1_14_vars": [0, 1, 2, 3, 4, 5, 6],
            "rzb_dcocrfnt:deco_s5_p1_01_vars": [0, 1, 2, 3, 4, 5, 6, 7],
            "rzb_dcocrfnt:deco_s5_p1_02_vars": [0, 1, 2, 3, 4, 5, 6, 7],
            "rzb_dcocrfnt:deco_s5_p1_03_vars": [0, 1, 2, 3, 4, 5, 6, 7],
            "rzb_dcocrfnt:deco_s5_p1_04_vars": [0, 1, 2, 3, 4, 5, 6, 7],
            "rzb_dcocrfnt:deco_s5_p1_05_vars": [0, 1, 2, 3, 4, 5, 6, 7],
        };
        const blockVariation = blockVariations[block.typeId];
        if (blockVariation) {
            system.runTimeout(() => {
                if (!checkAndSetCooldown(player,4)) return;
                player.playAnimation('animation.rzb_dcocrfnt.interact');
                world.playSound("mob.sheep.shear", block.location, soundOptions1);
                world.playSound("block.beehive.shear", block.location, soundOptions2);
                const nextVariation = (blockState + 1) % blockVariation.length;
                block.setPermutation(block.permutation.withState("rzb_dcocrfnt:variation", nextVariation));
            }, 1);
        }

        // Indvar blocks

        const blockTransitions = {
            "rzb_dcocrfnt:deco_s1_p1_01": "rzb_dcocrfnt:deco_s1_p1_02",
            "rzb_dcocrfnt:deco_s1_p1_02": "rzb_dcocrfnt:deco_s1_p1_03",
            "rzb_dcocrfnt:deco_s1_p1_03": "rzb_dcocrfnt:deco_s1_p1_04",
            "rzb_dcocrfnt:deco_s1_p1_04": "rzb_dcocrfnt:deco_s1_p1_01",

            "rzb_dcocrfnt:deco_s1_p1_05": "rzb_dcocrfnt:deco_s1_p1_06",
            "rzb_dcocrfnt:deco_s1_p1_06": "rzb_dcocrfnt:deco_s1_p1_07",
            "rzb_dcocrfnt:deco_s1_p1_07": "rzb_dcocrfnt:deco_s1_p1_05",
            
            "rzb_dcocrfnt:deco_s1_p1_10": "rzb_dcocrfnt:deco_s1_p1_11",
            "rzb_dcocrfnt:deco_s1_p1_11": "rzb_dcocrfnt:deco_s1_p1_12",
            "rzb_dcocrfnt:deco_s1_p1_12": "rzb_dcocrfnt:deco_s1_p1_10",

            "rzb_dcocrfnt:deco_s1_p1_13": "rzb_dcocrfnt:deco_s1_p1_14",
            "rzb_dcocrfnt:deco_s1_p1_14": "rzb_dcocrfnt:deco_s1_p1_15",
            "rzb_dcocrfnt:deco_s1_p1_15": "rzb_dcocrfnt:deco_s1_p1_13",

            "rzb_dcocrfnt:deco_s1_p1_16": "rzb_dcocrfnt:deco_s1_p1_17",
            "rzb_dcocrfnt:deco_s1_p1_17": "rzb_dcocrfnt:deco_s1_p1_16",

            "rzb_dcocrfnt:deco_s1_p1_18": "rzb_dcocrfnt:deco_s1_p1_19",
            "rzb_dcocrfnt:deco_s1_p1_19": "rzb_dcocrfnt:deco_s1_p1_18",

            "rzb_dcocrfnt:deco_s1_p1_20": "rzb_dcocrfnt:deco_s1_p1_21",
            "rzb_dcocrfnt:deco_s1_p1_21": "rzb_dcocrfnt:deco_s1_p1_20",

            "rzb_dcocrfnt:deco_s2_p1_16": "rzb_dcocrfnt:deco_s2_p1_17",
            "rzb_dcocrfnt:deco_s2_p1_17": "rzb_dcocrfnt:deco_s2_p1_16",

            "rzb_dcocrfnt:deco_s2_p1_18": "rzb_dcocrfnt:deco_s2_p1_19",
            "rzb_dcocrfnt:deco_s2_p1_19": "rzb_dcocrfnt:deco_s2_p1_18",

            "rzb_dcocrfnt:deco_s2_p1_20": "rzb_dcocrfnt:deco_s2_p1_21",
            "rzb_dcocrfnt:deco_s2_p1_21": "rzb_dcocrfnt:deco_s2_p1_20",

            "rzb_dcocrfnt:deco_s2_p1_22": "rzb_dcocrfnt:deco_s2_p1_23",
            "rzb_dcocrfnt:deco_s2_p1_23": "rzb_dcocrfnt:deco_s2_p1_22",

            "rzb_dcocrfnt:deco_s2_p1_24": "rzb_dcocrfnt:deco_s2_p1_25",
            "rzb_dcocrfnt:deco_s2_p1_25": "rzb_dcocrfnt:deco_s2_p1_24",

            "rzb_dcocrfnt:deco_s3_p1_01": "rzb_dcocrfnt:deco_s3_p1_02",
            "rzb_dcocrfnt:deco_s3_p1_02": "rzb_dcocrfnt:deco_s3_p1_01",

            "rzb_dcocrfnt:deco_s3_p1_03": "rzb_dcocrfnt:deco_s3_p1_04",
            "rzb_dcocrfnt:deco_s3_p1_04": "rzb_dcocrfnt:deco_s3_p1_03",

            "rzb_dcocrfnt:deco_s3_p1_05": "rzb_dcocrfnt:deco_s3_p1_06",
            "rzb_dcocrfnt:deco_s3_p1_06": "rzb_dcocrfnt:deco_s3_p1_05",

            "rzb_dcocrfnt:deco_s3_p1_09": "rzb_dcocrfnt:deco_s3_p1_10",
            "rzb_dcocrfnt:deco_s3_p1_10": "rzb_dcocrfnt:deco_s3_p1_09",

            "rzb_dcocrfnt:deco_s3_p1_11": "rzb_dcocrfnt:deco_s3_p1_12",
            "rzb_dcocrfnt:deco_s3_p1_12": "rzb_dcocrfnt:deco_s3_p1_11",

            "rzb_dcocrfnt:deco_s3_p1_13": "rzb_dcocrfnt:deco_s3_p1_14",
            "rzb_dcocrfnt:deco_s3_p1_14": "rzb_dcocrfnt:deco_s3_p1_13",

            "rzb_dcocrfnt:deco_s4_p1_13": "rzb_dcocrfnt:deco_s4_p1_20",
            "rzb_dcocrfnt:deco_s4_p1_20": "rzb_dcocrfnt:deco_s4_p1_13",

            "rzb_dcocrfnt:deco_s4_p1_14": "rzb_dcocrfnt:deco_s4_p1_21",
            "rzb_dcocrfnt:deco_s4_p1_21": "rzb_dcocrfnt:deco_s4_p1_14",

            "rzb_dcocrfnt:deco_s4_p1_15": "rzb_dcocrfnt:deco_s4_p1_22",
            "rzb_dcocrfnt:deco_s4_p1_22": "rzb_dcocrfnt:deco_s4_p1_15",

            "rzb_dcocrfnt:deco_s4_p1_16": "rzb_dcocrfnt:deco_s4_p1_23",
            "rzb_dcocrfnt:deco_s4_p1_23": "rzb_dcocrfnt:deco_s4_p1_16",

            "rzb_dcocrfnt:deco_s4_p1_17": "rzb_dcocrfnt:deco_s4_p1_24",
            "rzb_dcocrfnt:deco_s4_p1_24": "rzb_dcocrfnt:deco_s4_p1_17",

            "rzb_dcocrfnt:deco_s4_p1_18": "rzb_dcocrfnt:deco_s4_p1_25",
            "rzb_dcocrfnt:deco_s4_p1_25": "rzb_dcocrfnt:deco_s4_p1_18",

            "rzb_dcocrfnt:deco_s4_p1_19": "rzb_dcocrfnt:deco_s4_p1_26",
            "rzb_dcocrfnt:deco_s4_p1_26": "rzb_dcocrfnt:deco_s4_p1_19",

            "rzb_dcocrfnt:deco_s4_p2_03": "rzb_dcocrfnt:deco_s4_p2_08",
            "rzb_dcocrfnt:deco_s4_p2_08": "rzb_dcocrfnt:deco_s4_p2_03",

            "rzb_dcocrfnt:deco_s4_p2_04": "rzb_dcocrfnt:deco_s4_p2_09",
            "rzb_dcocrfnt:deco_s4_p2_09": "rzb_dcocrfnt:deco_s4_p2_04",

            "rzb_dcocrfnt:deco_s4_p2_05": "rzb_dcocrfnt:deco_s4_p2_10",
            "rzb_dcocrfnt:deco_s4_p2_10": "rzb_dcocrfnt:deco_s4_p2_05",

            "rzb_dcocrfnt:deco_s4_p2_06": "rzb_dcocrfnt:deco_s4_p2_11",
            "rzb_dcocrfnt:deco_s4_p2_11": "rzb_dcocrfnt:deco_s4_p2_06",

            "rzb_dcocrfnt:deco_s4_p2_07": "rzb_dcocrfnt:deco_s4_p2_12",
            "rzb_dcocrfnt:deco_s4_p2_12": "rzb_dcocrfnt:deco_s4_p2_07",

            "rzb_dcocrfnt:deco_s4_p2_13": "rzb_dcocrfnt:deco_s4_p2_17",
            "rzb_dcocrfnt:deco_s4_p2_17": "rzb_dcocrfnt:deco_s4_p2_13",

            "rzb_dcocrfnt:deco_s4_p2_14": "rzb_dcocrfnt:deco_s4_p2_18",
            "rzb_dcocrfnt:deco_s4_p2_18": "rzb_dcocrfnt:deco_s4_p2_14",

            "rzb_dcocrfnt:deco_s4_p2_15": "rzb_dcocrfnt:deco_s4_p2_19",
            "rzb_dcocrfnt:deco_s4_p2_19": "rzb_dcocrfnt:deco_s4_p2_15",

            "rzb_dcocrfnt:deco_s4_p2_16": "rzb_dcocrfnt:deco_s4_p2_20",
            "rzb_dcocrfnt:deco_s4_p2_20": "rzb_dcocrfnt:deco_s4_p2_16",

            "rzb_dcocrfnt:deco_s5_p1_09": "rzb_dcocrfnt:deco_s5_p1_10",
            "rzb_dcocrfnt:deco_s5_p1_10": "rzb_dcocrfnt:deco_s5_p1_11",
            "rzb_dcocrfnt:deco_s5_p1_11": "rzb_dcocrfnt:deco_s5_p1_09",

            "rzb_dcocrfnt:deco_s6_p1_13": "rzb_dcocrfnt:deco_s6_p1_14",
            "rzb_dcocrfnt:deco_s6_p1_14": "rzb_dcocrfnt:deco_s6_p1_15",
            "rzb_dcocrfnt:deco_s6_p1_15": "rzb_dcocrfnt:deco_s6_p1_16",
            "rzb_dcocrfnt:deco_s6_p1_16": "rzb_dcocrfnt:deco_s6_p1_17",
            "rzb_dcocrfnt:deco_s6_p1_17": "rzb_dcocrfnt:deco_s6_p1_13",

            "rzb_dcocrfnt:deco_s7_p1_01": "rzb_dcocrfnt:deco_s7_p1_02",
            "rzb_dcocrfnt:deco_s7_p1_02": "rzb_dcocrfnt:deco_s7_p1_03",
            "rzb_dcocrfnt:deco_s7_p1_03": "rzb_dcocrfnt:deco_s7_p1_01",

            "rzb_dcocrfnt:deco_s8_p1_01": "rzb_dcocrfnt:deco_s8_p1_02",
            "rzb_dcocrfnt:deco_s8_p1_02": "rzb_dcocrfnt:deco_s8_p1_01",

            "rzb_dcocrfnt:deco_s8_p1_03": "rzb_dcocrfnt:deco_s8_p1_04",
            "rzb_dcocrfnt:deco_s8_p1_04": "rzb_dcocrfnt:deco_s8_p1_03",

            "rzb_dcocrfnt:deco_s8_p1_05": "rzb_dcocrfnt:deco_s8_p1_06",
            "rzb_dcocrfnt:deco_s8_p1_06": "rzb_dcocrfnt:deco_s8_p1_05",

            "rzb_dcocrfnt:deco_s8_p1_07": "rzb_dcocrfnt:deco_s8_p1_08",
            "rzb_dcocrfnt:deco_s8_p1_08": "rzb_dcocrfnt:deco_s8_p1_07",
            
            "rzb_dcocrfnt:deco_s8_p1_09": "rzb_dcocrfnt:deco_s8_p1_10",
            "rzb_dcocrfnt:deco_s8_p1_10": "rzb_dcocrfnt:deco_s8_p1_09"
        };

        const nextBlockType = blockTransitions[block.typeId];
        if (nextBlockType) {
            system.runTimeout(() => {
                if (!checkAndSetCooldown(player,4)) return;
                player.playAnimation('animation.rzb_dcocrfnt.interact');
                world.playSound("mob.sheep.shear", block.location, soundOptions1);
                world.playSound("block.beehive.shear", block.location, soundOptions2);
                block.setPermutation(BlockPermutation.resolve(nextBlockType, { "minecraft:cardinal_direction": blockOrientation }));
            }, 1);
        }
    }
})

// Clipping blocks LEFT, and Removing parts blocks

world.beforeEvents.playerBreakBlock.subscribe((eventData) => {
    const {player, itemStack, block} = eventData;
    const blockOrientation = block.permutation.getState("minecraft:cardinal_direction");
    const blockState = block.permutation.getState("rzb_dcocrfnt:variation");

    // Clipping blocks LEFT

    // Vars blocks

    const blockVariationsMap = {
        2: [
            "rzb_dcocrfnt:deco_s11_p1_12_vars",
            "rzb_dcocrfnt:deco_s11_p1_16_vars",
            "rzb_dcocrfnt:deco_s11_p1_20_vars",
            "rzb_dcocrfnt:deco_s11_p1_24_vars",
            "rzb_dcocrfnt:deco_s2_p1_01_vars",
            "rzb_dcocrfnt:deco_s2_p1_03_vars",
            "rzb_dcocrfnt:deco_s2_p1_04_vars",
            "rzb_dcocrfnt:deco_s2_p1_06_vars",
            "rzb_dcocrfnt:deco_s2_p1_07_vars",
            "rzb_dcocrfnt:deco_s2_p1_09_vars",
            "rzb_dcocrfnt:deco_s2_p1_10_vars",
            "rzb_dcocrfnt:deco_s2_p1_12_vars",
            "rzb_dcocrfnt:deco_s2_p1_13_vars",
            "rzb_dcocrfnt:deco_s2_p1_15_vars"
        ],
        3: [
            "rzb_dcocrfnt:deco_s11_p1_14_vars",
            "rzb_dcocrfnt:deco_s11_p1_15_vars",
            "rzb_dcocrfnt:deco_s11_p1_18_vars",
            "rzb_dcocrfnt:deco_s11_p1_19_vars",
            "rzb_dcocrfnt:deco_s11_p1_22_vars",
            "rzb_dcocrfnt:deco_s11_p1_23_vars",
            "rzb_dcocrfnt:deco_s11_p1_26_vars",
            "rzb_dcocrfnt:deco_s11_p1_27_vars"
        ],
        4: [
            "rzb_dcocrfnt:deco_s11_p1_08_vars",
            "rzb_dcocrfnt:deco_s11_p1_09_vars",
            "rzb_dcocrfnt:deco_s11_p1_10_vars",
            "rzb_dcocrfnt:deco_s11_p1_11_vars",
            "rzb_dcocrfnt:deco_s11_p1_13_vars",
            "rzb_dcocrfnt:deco_s11_p1_17_vars",
            "rzb_dcocrfnt:deco_s11_p1_21_vars",
            "rzb_dcocrfnt:deco_s11_p1_25_vars",
            "rzb_dcocrfnt:deco_s5_p1_13_vars",
            "rzb_dcocrfnt:deco_s5_p1_14_vars",
            "rzb_dcocrfnt:deco_s5_p1_15_vars",
            "rzb_dcocrfnt:deco_s5_p1_16_vars",
            "rzb_dcocrfnt:deco_s5_p1_17_vars",
            "rzb_dcocrfnt:deco_s5_p1_18_vars",
            "rzb_dcocrfnt:deco_s5_p1_19_vars",
            "rzb_dcocrfnt:deco_s5_p1_20_vars",
            "rzb_dcocrfnt:deco_s5_p1_21_vars"
        ],
        5: [
            "rzb_dcocrfnt:deco_s2_p1_26_vars",
            "rzb_dcocrfnt:deco_s2_p2_02_vars",
            "rzb_dcocrfnt:deco_s2_p2_06_vars",
            "rzb_dcocrfnt:deco_s2_p2_10_vars",
            "rzb_dcocrfnt:deco_s2_p2_14_vars"
        ],
        6: [
            "rzb_dcocrfnt:deco_s2_p1_02_vars",
            "rzb_dcocrfnt:deco_s2_p1_05_vars",
            "rzb_dcocrfnt:deco_s2_p1_08_vars",
            "rzb_dcocrfnt:deco_s2_p1_11_vars",
            "rzb_dcocrfnt:deco_s2_p1_14_vars"
        ],
        7: [
            "rzb_dcocrfnt:deco_s5_p1_01_vars",
            "rzb_dcocrfnt:deco_s5_p1_02_vars",
            "rzb_dcocrfnt:deco_s5_p1_03_vars",
            "rzb_dcocrfnt:deco_s5_p1_04_vars",
            "rzb_dcocrfnt:deco_s5_p1_05_vars"
        ]
    };
    if (Object.values(blockVariationsMap).flat().includes(block.typeId) && itemStack?.typeId ==  "rzb_dcocrfnt:hedge_clipper") {
        eventData.cancel = true;
        for (const [variation, typeIds] of Object.entries(blockVariationsMap)) {
            if (typeIds.includes(block.typeId)) {
                system.runTimeout(() => {
                    if (!checkAndSetCooldown(player,4)) return;
                    player.playAnimation('animation.rzb_dcocrfnt.interact');
                    world.playSound("mob.sheep.shear", block.location, soundOptions1);
                    world.playSound("block.beehive.shear", block.location, soundOptions2);
                    const nextState = blockState === 0 ? parseInt(variation) : blockState - 1;
                    block.setPermutation(block.permutation.withState("rzb_dcocrfnt:variation", nextState));
                }, 1);
                break;
            }
        }
    }

    // Indvar blocks

    const blockPermutations = {
        "rzb_dcocrfnt:deco_s1_p1_01": "rzb_dcocrfnt:deco_s1_p1_04",
        "rzb_dcocrfnt:deco_s1_p1_02": "rzb_dcocrfnt:deco_s1_p1_01",
        "rzb_dcocrfnt:deco_s1_p1_03": "rzb_dcocrfnt:deco_s1_p1_02",
        "rzb_dcocrfnt:deco_s1_p1_04": "rzb_dcocrfnt:deco_s1_p1_03",

        "rzb_dcocrfnt:deco_s1_p1_05": "rzb_dcocrfnt:deco_s1_p1_07",
        "rzb_dcocrfnt:deco_s1_p1_06": "rzb_dcocrfnt:deco_s1_p1_05",
        "rzb_dcocrfnt:deco_s1_p1_07": "rzb_dcocrfnt:deco_s1_p1_06",

        "rzb_dcocrfnt:deco_s1_p1_10": "rzb_dcocrfnt:deco_s1_p1_12",
        "rzb_dcocrfnt:deco_s1_p1_11": "rzb_dcocrfnt:deco_s1_p1_10",
        "rzb_dcocrfnt:deco_s1_p1_12": "rzb_dcocrfnt:deco_s1_p1_11",

        "rzb_dcocrfnt:deco_s1_p1_13": "rzb_dcocrfnt:deco_s1_p1_15",
        "rzb_dcocrfnt:deco_s1_p1_14": "rzb_dcocrfnt:deco_s1_p1_13",
        "rzb_dcocrfnt:deco_s1_p1_15": "rzb_dcocrfnt:deco_s1_p1_14",

        "rzb_dcocrfnt:deco_s1_p1_16": "rzb_dcocrfnt:deco_s1_p1_17",
        "rzb_dcocrfnt:deco_s1_p1_17": "rzb_dcocrfnt:deco_s1_p1_16",

        "rzb_dcocrfnt:deco_s1_p1_18": "rzb_dcocrfnt:deco_s1_p1_19",
        "rzb_dcocrfnt:deco_s1_p1_19": "rzb_dcocrfnt:deco_s1_p1_18",

        "rzb_dcocrfnt:deco_s1_p1_20": "rzb_dcocrfnt:deco_s1_p1_21",
        "rzb_dcocrfnt:deco_s1_p1_21": "rzb_dcocrfnt:deco_s1_p1_20",

        "rzb_dcocrfnt:deco_s2_p1_16": "rzb_dcocrfnt:deco_s2_p1_17",
        "rzb_dcocrfnt:deco_s2_p1_17": "rzb_dcocrfnt:deco_s2_p1_16",

        "rzb_dcocrfnt:deco_s2_p1_18": "rzb_dcocrfnt:deco_s2_p1_19",
        "rzb_dcocrfnt:deco_s2_p1_19": "rzb_dcocrfnt:deco_s2_p1_18",

        "rzb_dcocrfnt:deco_s2_p1_20": "rzb_dcocrfnt:deco_s2_p1_21",
        "rzb_dcocrfnt:deco_s2_p1_21": "rzb_dcocrfnt:deco_s2_p1_20",

        "rzb_dcocrfnt:deco_s2_p1_22": "rzb_dcocrfnt:deco_s2_p1_23",
        "rzb_dcocrfnt:deco_s2_p1_23": "rzb_dcocrfnt:deco_s2_p1_22",

        "rzb_dcocrfnt:deco_s2_p1_24": "rzb_dcocrfnt:deco_s2_p1_25",
        "rzb_dcocrfnt:deco_s2_p1_25": "rzb_dcocrfnt:deco_s2_p1_24",

        "rzb_dcocrfnt:deco_s3_p1_01": "rzb_dcocrfnt:deco_s3_p1_02",
        "rzb_dcocrfnt:deco_s3_p1_02": "rzb_dcocrfnt:deco_s3_p1_01",

        "rzb_dcocrfnt:deco_s3_p1_03": "rzb_dcocrfnt:deco_s3_p1_04",
        "rzb_dcocrfnt:deco_s3_p1_04": "rzb_dcocrfnt:deco_s3_p1_03",

        "rzb_dcocrfnt:deco_s3_p1_05": "rzb_dcocrfnt:deco_s3_p1_06",
        "rzb_dcocrfnt:deco_s3_p1_06": "rzb_dcocrfnt:deco_s3_p1_05",

        "rzb_dcocrfnt:deco_s3_p1_09": "rzb_dcocrfnt:deco_s3_p1_10",
        "rzb_dcocrfnt:deco_s3_p1_10": "rzb_dcocrfnt:deco_s3_p1_09",

        "rzb_dcocrfnt:deco_s3_p1_11": "rzb_dcocrfnt:deco_s3_p1_12",
        "rzb_dcocrfnt:deco_s3_p1_12": "rzb_dcocrfnt:deco_s3_p1_11",

        "rzb_dcocrfnt:deco_s3_p1_13": "rzb_dcocrfnt:deco_s3_p1_14",
        "rzb_dcocrfnt:deco_s3_p1_14": "rzb_dcocrfnt:deco_s3_p1_13",

        "rzb_dcocrfnt:deco_s4_p1_13": "rzb_dcocrfnt:deco_s4_p1_20",
        "rzb_dcocrfnt:deco_s4_p1_20": "rzb_dcocrfnt:deco_s4_p1_13",

        "rzb_dcocrfnt:deco_s4_p1_14": "rzb_dcocrfnt:deco_s4_p1_21",
        "rzb_dcocrfnt:deco_s4_p1_21": "rzb_dcocrfnt:deco_s4_p1_14",

        "rzb_dcocrfnt:deco_s4_p1_15": "rzb_dcocrfnt:deco_s4_p1_22",
        "rzb_dcocrfnt:deco_s4_p1_22": "rzb_dcocrfnt:deco_s4_p1_15",

        "rzb_dcocrfnt:deco_s4_p1_16": "rzb_dcocrfnt:deco_s4_p1_23",
        "rzb_dcocrfnt:deco_s4_p1_23": "rzb_dcocrfnt:deco_s4_p1_16",

        "rzb_dcocrfnt:deco_s4_p1_17": "rzb_dcocrfnt:deco_s4_p1_24",
        "rzb_dcocrfnt:deco_s4_p1_24": "rzb_dcocrfnt:deco_s4_p1_17",
        
        "rzb_dcocrfnt:deco_s4_p1_18": "rzb_dcocrfnt:deco_s4_p1_25",
        "rzb_dcocrfnt:deco_s4_p1_25": "rzb_dcocrfnt:deco_s4_p1_18",

        "rzb_dcocrfnt:deco_s4_p1_19": "rzb_dcocrfnt:deco_s4_p1_26",
        "rzb_dcocrfnt:deco_s4_p1_26": "rzb_dcocrfnt:deco_s4_p1_19",

        "rzb_dcocrfnt:deco_s4_p2_03": "rzb_dcocrfnt:deco_s4_p2_08",
        "rzb_dcocrfnt:deco_s4_p2_08": "rzb_dcocrfnt:deco_s4_p2_03",

        "rzb_dcocrfnt:deco_s4_p2_04": "rzb_dcocrfnt:deco_s4_p2_09",
        "rzb_dcocrfnt:deco_s4_p2_09": "rzb_dcocrfnt:deco_s4_p2_04",

        "rzb_dcocrfnt:deco_s4_p2_05": "rzb_dcocrfnt:deco_s4_p2_10",
        "rzb_dcocrfnt:deco_s4_p2_10": "rzb_dcocrfnt:deco_s4_p2_05",

        "rzb_dcocrfnt:deco_s4_p2_06": "rzb_dcocrfnt:deco_s4_p2_11",
        "rzb_dcocrfnt:deco_s4_p2_11": "rzb_dcocrfnt:deco_s4_p2_06",

        "rzb_dcocrfnt:deco_s4_p2_07": "rzb_dcocrfnt:deco_s4_p2_12",
        "rzb_dcocrfnt:deco_s4_p2_12": "rzb_dcocrfnt:deco_s4_p2_07",

        "rzb_dcocrfnt:deco_s4_p2_13": "rzb_dcocrfnt:deco_s4_p2_17",
        "rzb_dcocrfnt:deco_s4_p2_17": "rzb_dcocrfnt:deco_s4_p2_13",

        "rzb_dcocrfnt:deco_s4_p2_14": "rzb_dcocrfnt:deco_s4_p2_18",
        "rzb_dcocrfnt:deco_s4_p2_18": "rzb_dcocrfnt:deco_s4_p2_14",

        "rzb_dcocrfnt:deco_s4_p2_15": "rzb_dcocrfnt:deco_s4_p2_19",
        "rzb_dcocrfnt:deco_s4_p2_19": "rzb_dcocrfnt:deco_s4_p2_15",

        "rzb_dcocrfnt:deco_s4_p2_16": "rzb_dcocrfnt:deco_s4_p2_20",
        "rzb_dcocrfnt:deco_s4_p2_20": "rzb_dcocrfnt:deco_s4_p2_16",

        "rzb_dcocrfnt:deco_s5_p1_09": "rzb_dcocrfnt:deco_s5_p1_11",
        "rzb_dcocrfnt:deco_s5_p1_10": "rzb_dcocrfnt:deco_s5_p1_09",
        "rzb_dcocrfnt:deco_s5_p1_11": "rzb_dcocrfnt:deco_s5_p1_10",

        "rzb_dcocrfnt:deco_s6_p1_13": "rzb_dcocrfnt:deco_s6_p1_17",
        "rzb_dcocrfnt:deco_s6_p1_14": "rzb_dcocrfnt:deco_s6_p1_13",
        "rzb_dcocrfnt:deco_s6_p1_15": "rzb_dcocrfnt:deco_s6_p1_14",
        "rzb_dcocrfnt:deco_s6_p1_16": "rzb_dcocrfnt:deco_s6_p1_15",
        "rzb_dcocrfnt:deco_s6_p1_17": "rzb_dcocrfnt:deco_s6_p1_16",

        "rzb_dcocrfnt:deco_s7_p1_01": "rzb_dcocrfnt:deco_s7_p1_03",
        "rzb_dcocrfnt:deco_s7_p1_02": "rzb_dcocrfnt:deco_s7_p1_01",
        "rzb_dcocrfnt:deco_s7_p1_03": "rzb_dcocrfnt:deco_s7_p1_02",

        "rzb_dcocrfnt:deco_s8_p1_01": "rzb_dcocrfnt:deco_s8_p1_02",
        "rzb_dcocrfnt:deco_s8_p1_02": "rzb_dcocrfnt:deco_s8_p1_01",

        "rzb_dcocrfnt:deco_s8_p1_03": "rzb_dcocrfnt:deco_s8_p1_04",
        "rzb_dcocrfnt:deco_s8_p1_04": "rzb_dcocrfnt:deco_s8_p1_03",

        "rzb_dcocrfnt:deco_s8_p1_05": "rzb_dcocrfnt:deco_s8_p1_06",
        "rzb_dcocrfnt:deco_s8_p1_06": "rzb_dcocrfnt:deco_s8_p1_05",

        "rzb_dcocrfnt:deco_s8_p1_07": "rzb_dcocrfnt:deco_s8_p1_08",
        "rzb_dcocrfnt:deco_s8_p1_08": "rzb_dcocrfnt:deco_s8_p1_07",
        
        "rzb_dcocrfnt:deco_s8_p1_09": "rzb_dcocrfnt:deco_s8_p1_10",
        "rzb_dcocrfnt:deco_s8_p1_10": "rzb_dcocrfnt:deco_s8_p1_09"
    };
    if (blockPermutations[block.typeId] && itemStack?.typeId == "rzb_dcocrfnt:hedge_clipper") {
        eventData.cancel = true;
        const nextBlockTypeTwo = blockPermutations[block.typeId];
        if (nextBlockTypeTwo) {
            system.runTimeout(() => {
                if (!checkAndSetCooldown(player,4)) return;
                player.playAnimation('animation.rzb_dcocrfnt.interact');
                world.playSound("mob.sheep.shear", block.location, soundOptions1);
                world.playSound("block.beehive.shear", block.location, soundOptions2);
                block.setPermutation(BlockPermutation.resolve(nextBlockTypeTwo, { "minecraft:cardinal_direction": blockOrientation }));
            }, 1);
        }
    }

    // Removing parts blocks

    const location = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
    const arrayOfStringsOne = [
        "rzb_dcocrfnt:deco_s2_p1_28_parts",
        "rzb_dcocrfnt:deco_s2_p2_01_parts",
        "rzb_dcocrfnt:deco_s2_p2_04_parts",
        "rzb_dcocrfnt:deco_s2_p2_05_parts",
        "rzb_dcocrfnt:deco_s2_p2_08_parts",
        "rzb_dcocrfnt:deco_s2_p2_09_parts",
        "rzb_dcocrfnt:deco_s2_p2_12_parts",
        "rzb_dcocrfnt:deco_s2_p2_13_parts",
        "rzb_dcocrfnt:deco_s2_p2_16_parts",
        "rzb_dcocrfnt:deco_s2_p2_17_parts",
        "rzb_dcocrfnt:deco_s11_p1_06",
        "rzb_dcocrfnt:deco_s11_p1_07"
    ];
    const resultOne = arrayOfStringsOne.some((str) => block.permutation.matches(str));
    if (resultOne) {
        system.runTimeout(() => {
            location.setPermutation(BlockPermutation.resolve('minecraft:air'));
        }, 0);
    }

    const blockCardinalDir = block.permutation.getState("minecraft:cardinal_direction");

    const directions = {
        south: [
            { x: 0, y: 0, z: 1 },
            { x: 0, y: 0, z: -1 },
            { x: -1, y: 0, z: -1 },
            { x: -1, y: 0, z: 0 },
            { x: 0, y: 1, z: -1 }
        ],
        west: [
            { x: -1, y: 0, z: 0 },
            { x: 1, y: 0, z: 0 },
            { x: 1, y: 0, z: -1 },
            { x: 0, y: 0, z: -1 },
            { x: 1, y: 1, z: 0 }
        ],
        north: [
            { x: 0, y: 0, z: -1 },
            { x: 0, y: 0, z: 1 },
            { x: 1, y: 0, z: 1 },
            { x: 1, y: 0, z: 0 },
            { x: 0, y: 1, z: 1 }
        ],
        east: [
            { x: 1, y: 0, z: 0 },
            { x: -1, y: 0, z: 0 },
            { x: -1, y: 0, z: 1 },
            { x: 0, y: 0, z: 1 },
            { x: -1, y: 1, z: 0 }
        ]
    };

    const removeBlocks = (offsets, delay = 1) => {
        system.runTimeout(() => {
            offsets.forEach(offset => {
                const loc = block.dimension.getBlock({
                    x: block.location.x + offset.x,
                    y: block.location.y + offset.y,
                    z: block.location.z + offset.z
                });
                loc.setPermutation(BlockPermutation.resolve('minecraft:air'));
            });
        }, delay);
    };

    const arrayOfStringsTwo = [
        "rzb_dcocrfnt:deco_s10_p1_18_part1",
        "rzb_dcocrfnt:deco_s10_p1_19_part1",
        "rzb_dcocrfnt:deco_s10_p1_20_part1",
        "rzb_dcocrfnt:deco_s10_p1_21_part1"
    ];
    const resultTwo = arrayOfStringsTwo.some(str => block.permutation.matches(str));
    if (resultTwo) {
        removeBlocks([directions[blockCardinalDir][0]]);
    }

    const resultThree = block.permutation.matches("rzb_dcocrfnt:deco_s3_p1_15_parts");
    if (resultThree) {
        removeBlocks(directions[blockCardinalDir]);
    }

    const resultFour = block.permutation.matches("rzb_dcocrfnt:deco_s3_p1_16_parts");
    if (resultFour) {
        removeBlocks([{ x: 0, y: 1, z: 0 }, { x: 0, y: 2, z: 0 }], 0);
        removeBlocks(directions[blockCardinalDir]);
    }

    const arrayOfStringsFive = [
        "rzb_dcocrfnt:deco_s3_p1_17_parts",
        "rzb_dcocrfnt:deco_s3_p1_18_parts",
        "rzb_dcocrfnt:deco_s3_p1_19_parts"
    ];
    if (arrayOfStringsFive.some(str => block.permutation.matches(str))) {
        removeBlocks([{ x: 0, y: 1, z: 0 }], 0);
        removeBlocks(directions[blockCardinalDir]);
    }
})

// Parts blocks PLACE (with custom components)

const directions = ["north", "east", "south", "west"];
const directionsTwo = ["south", "west", "north", "east"];

world.beforeEvents.worldInitialize.subscribe((data) => {
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s2_p1_28_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];
        
            if(location.typeId == "minecraft:air" || location.typeId == "minecraft:water" || location.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p1_28_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p1_28_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
            } else {
                eventData.cancel = true;
                player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                player.playSound("note.didgeridoo", soundOptions3);
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s2_p2_01_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];
        
            if(location.typeId == "minecraft:air" || location.typeId == "minecraft:water" || location.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_01_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_01_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
            } else {
                eventData.cancel = true;
                player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                player.playSound("note.didgeridoo", soundOptions3);
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s2_p2_04_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];
        
            if(location.typeId == "minecraft:air" || location.typeId == "minecraft:water" || location.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_04_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_04_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
            } else {
                eventData.cancel = true;
                player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                player.playSound("note.didgeridoo", soundOptions3);
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s2_p2_05_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];
        
            if(location.typeId == "minecraft:air" || location.typeId == "minecraft:water" || location.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_05_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_05_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
            } else {
                eventData.cancel = true;
                player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                player.playSound("note.didgeridoo", soundOptions3);
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s2_p2_08_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];
        
            if(location.typeId == "minecraft:air" || location.typeId == "minecraft:water" || location.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_08_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_08_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
            } else {
                eventData.cancel = true;
                player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                player.playSound("note.didgeridoo", soundOptions3);
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s2_p2_09_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];
        
            if(location.typeId == "minecraft:air" || location.typeId == "minecraft:water" || location.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_09_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_09_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
            } else {
                eventData.cancel = true;
                player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                player.playSound("note.didgeridoo", soundOptions3);
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s2_p2_12_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];
        
            if(location.typeId == "minecraft:air" || location.typeId == "minecraft:water" || location.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_12_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_12_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
            } else {
                eventData.cancel = true;
                player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                player.playSound("note.didgeridoo", soundOptions3);
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s2_p2_13_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];
        
            if(location.typeId == "minecraft:air" || location.typeId == "minecraft:water" || location.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_13_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_13_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
            } else {
                eventData.cancel = true;
                player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                player.playSound("note.didgeridoo", soundOptions3);
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s2_p2_16_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];
        
            if(location.typeId == "minecraft:air" || location.typeId == "minecraft:water" || location.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_16_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
            } else {
                eventData.cancel = true;
                player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                player.playSound("note.didgeridoo", soundOptions3);
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s2_p2_17_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];
        
            if(location.typeId == "minecraft:air" || location.typeId == "minecraft:water" || location.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_17_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s2_p2_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
            } else {
                eventData.cancel = true;
                player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                player.playSound("note.didgeridoo", soundOptions3);
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s11_p1_06", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];
        
            if(location.typeId == "minecraft:air" || location.typeId == "minecraft:water" || location.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s11_p1_06', { "minecraft:cardinal_direction": playerDirection }));
                    location.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:invisible_block_4', { "minecraft:cardinal_direction": playerDirectionTwo }));
            } else {
                eventData.cancel = true;
                player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                player.playSound("note.didgeridoo", soundOptions3);
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s11_p1_07", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];
        
            if(location.typeId == "minecraft:air" || location.typeId == "minecraft:water" || location.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s11_p1_07', { "minecraft:cardinal_direction": playerDirection }));
                    location.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:invisible_block_4', { "minecraft:cardinal_direction": playerDirectionTwo }));
            } else {
                eventData.cancel = true;
                player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                player.playSound("note.didgeridoo", soundOptions3);
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s10_p1_18_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1 });
            const location_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z });
            const location_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1 });
            const location_4 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];

            if(playerDirection == "north") {
                if(location_1.typeId == "minecraft:air" || location_1.typeId == "minecraft:water" || location_1.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "east")  {
                if(location_2.typeId == "minecraft:air" || location_2.typeId == "minecraft:water" || location_2.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "south")  {
                if(location_3.typeId == "minecraft:air" || location_3.typeId == "minecraft:water" || location_3.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "west")  {
                if(location_4.typeId == "minecraft:air" || location_4.typeId == "minecraft:water" || location_4.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s10_p1_19_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1 });
            const location_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z });
            const location_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1 });
            const location_4 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];

            if(playerDirection == "north") {
                if(location_1.typeId == "minecraft:air" || location_1.typeId == "minecraft:water" || location_1.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "east")  {
                if(location_2.typeId == "minecraft:air" || location_2.typeId == "minecraft:water" || location_2.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "south")  {
                if(location_3.typeId == "minecraft:air" || location_3.typeId == "minecraft:water" || location_3.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "west")  {
                if(location_4.typeId == "minecraft:air" || location_4.typeId == "minecraft:water" || location_4.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s10_p1_20_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1 });
            const location_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z });
            const location_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1 });
            const location_4 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];

            if(playerDirection == "north") {
                if(location_1.typeId == "minecraft:air" || location_1.typeId == "minecraft:water" || location_1.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "east")  {
                if(location_2.typeId == "minecraft:air" || location_2.typeId == "minecraft:water" || location_2.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "south")  {
                if(location_3.typeId == "minecraft:air" || location_3.typeId == "minecraft:water" || location_3.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "west")  {
                if(location_4.typeId == "minecraft:air" || location_4.typeId == "minecraft:water" || location_4.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s10_p1_21_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            const location_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1 });
            const location_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z });
            const location_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1 });
            const location_4 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z });
            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];

            if(playerDirection == "north") {
                if(location_1.typeId == "minecraft:air" || location_1.typeId == "minecraft:water" || location_1.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "east")  {
                if(location_2.typeId == "minecraft:air" || location_2.typeId == "minecraft:water" || location_2.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "south")  {
                if(location_3.typeId == "minecraft:air" || location_3.typeId == "minecraft:water" || location_3.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "west")  {
                if(location_4.typeId == "minecraft:air" || location_4.typeId == "minecraft:water" || location_4.typeId == "minecraft:flowing_water") {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18_part1', { "minecraft:cardinal_direction": playerDirection }));
                    location_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s10_p1_18-19-20-21_part2', { "minecraft:cardinal_direction": playerDirectionTwo }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s3_p1_15_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;

            const location_1_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1});
            const location_1_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z - 1 });
            const location_1_3 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z });

            const location_2_1 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z});
            const location_2_2 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z - 1 });
            const location_2_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1 });

            const location_3_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1});
            const location_3_2 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z + 1 });
            const location_3_3 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z });

            const location_4_1 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z});
            const location_4_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z + 1 });
            const location_4_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1 });

            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];

            if(playerDirection == "north") {
                if((location_1_1.typeId == "minecraft:air" || location_1_1.typeId == "minecraft:water" || location_1_1.typeId == "minecraft:flowing_water") && (location_1_2.typeId == "minecraft:air" || location_1_2.typeId == "minecraft:water" || location_1_2.typeId == "minecraft:flowing_water") && (location_1_3.typeId == "minecraft:air" || location_1_3.typeId == "minecraft:water" || location_1_3.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_1_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_1_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_1_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "east")  {
                if((location_2_1.typeId == "minecraft:air" || location_2_1.typeId == "minecraft:water" || location_2_1.typeId == "minecraft:flowing_water") && (location_2_2.typeId == "minecraft:air" || location_2_2.typeId == "minecraft:water" || location_2_2.typeId == "minecraft:flowing_water") && (location_2_3.typeId == "minecraft:air" || location_2_3.typeId == "minecraft:water" || location_2_3.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_2_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_2_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_2_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "south")  {
                if((location_3_1.typeId == "minecraft:air" || location_3_1.typeId == "minecraft:water" || location_3_1.typeId == "minecraft:flowing_water") && (location_3_2.typeId == "minecraft:air" || location_3_2.typeId == "minecraft:water" || location_3_2.typeId == "minecraft:flowing_water") && (location_3_3.typeId == "minecraft:air" || location_3_3.typeId == "minecraft:water" || location_3_3.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_3_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_3_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_3_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "west")  {
                if((location_4_1.typeId == "minecraft:air" || location_4_1.typeId == "minecraft:water" || location_4_1.typeId == "minecraft:flowing_water") && (location_4_2.typeId == "minecraft:air" || location_4_2.typeId == "minecraft:water" || location_4_2.typeId == "minecraft:flowing_water") && (location_4_3.typeId == "minecraft:air" || location_4_3.typeId == "minecraft:water" || location_4_3.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_4_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_4_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_4_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_15_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s3_p1_16_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            
            const location_1_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1});
            const location_1_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z - 1 });
            const location_1_3 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z });

            const location_2_1 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z});
            const location_2_2 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z - 1 });
            const location_2_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1 });

            const location_3_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1});
            const location_3_2 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z + 1 });
            const location_3_3 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z });

            const location_4_1 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z});
            const location_4_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z + 1 });
            const location_4_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1 });
            
            const location_1234_4 = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });
            const location_1234_5 = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 2, z: block.location.z });

            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];

            if(playerDirection == "north") {
                if((location_1_1.typeId == "minecraft:air" || location_1_1.typeId == "minecraft:water" || location_1_1.typeId == "minecraft:flowing_water") && (location_1_2.typeId == "minecraft:air" || location_1_2.typeId == "minecraft:water" || location_1_2.typeId == "minecraft:flowing_water") && (location_1_3.typeId == "minecraft:air" || location_1_3.typeId == "minecraft:water" || location_1_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_1234_5.typeId == "minecraft:air" || location_1234_5.typeId == "minecraft:water" || location_1234_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_1_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_1_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_1_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_1234_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "east")  {
                if((location_2_1.typeId == "minecraft:air" || location_2_1.typeId == "minecraft:water" || location_2_1.typeId == "minecraft:flowing_water") && (location_2_2.typeId == "minecraft:air" || location_2_2.typeId == "minecraft:water" || location_2_2.typeId == "minecraft:flowing_water") && (location_2_3.typeId == "minecraft:air" || location_2_3.typeId == "minecraft:water" || location_2_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_1234_5.typeId == "minecraft:air" || location_1234_5.typeId == "minecraft:water" || location_1234_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_2_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_2_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_2_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_1234_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "south")  {
                if((location_3_1.typeId == "minecraft:air" || location_3_1.typeId == "minecraft:water" || location_3_1.typeId == "minecraft:flowing_water") && (location_3_2.typeId == "minecraft:air" || location_3_2.typeId == "minecraft:water" || location_3_2.typeId == "minecraft:flowing_water") && (location_3_3.typeId == "minecraft:air" || location_3_3.typeId == "minecraft:water" || location_3_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_1234_5.typeId == "minecraft:air" || location_1234_5.typeId == "minecraft:water" || location_1234_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_3_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_3_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_3_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_1234_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "west")  {
                if((location_4_1.typeId == "minecraft:air" || location_4_1.typeId == "minecraft:water" || location_4_1.typeId == "minecraft:flowing_water") && (location_4_2.typeId == "minecraft:air" || location_4_2.typeId == "minecraft:water" || location_4_2.typeId == "minecraft:flowing_water") && (location_4_3.typeId == "minecraft:air" || location_4_3.typeId == "minecraft:water" || location_4_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_1234_5.typeId == "minecraft:air" || location_1234_5.typeId == "minecraft:water" || location_1234_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_4_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_4_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_4_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_1234_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_16_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s3_p1_17_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            
            const location_1_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1});
            const location_1_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z - 1 });
            const location_1_3 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z });
            const location_1_5 = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z - 1 });

            const location_2_1 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z});
            const location_2_2 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z - 1 });
            const location_2_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1 });
            const location_2_5 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y + 1, z: block.location.z });

            const location_3_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1});
            const location_3_2 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z + 1 });
            const location_3_3 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z });
            const location_3_5 = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z + 1 });

            const location_4_1 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z});
            const location_4_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z + 1 });
            const location_4_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1 });
            const location_4_5 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y + 1, z: block.location.z });
            
            const location_1234_4 = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });

            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];

            if(playerDirection == "north") {
                if((location_1_1.typeId == "minecraft:air" || location_1_1.typeId == "minecraft:water" || location_1_1.typeId == "minecraft:flowing_water") && (location_1_2.typeId == "minecraft:air" || location_1_2.typeId == "minecraft:water" || location_1_2.typeId == "minecraft:flowing_water") && (location_1_3.typeId == "minecraft:air" || location_1_3.typeId == "minecraft:water" || location_1_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_1_5.typeId == "minecraft:air" || location_1_5.typeId == "minecraft:water" || location_1_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_1_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_1_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_1_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_1_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "east")  {
                if((location_2_1.typeId == "minecraft:air" || location_2_1.typeId == "minecraft:water" || location_2_1.typeId == "minecraft:flowing_water") && (location_2_2.typeId == "minecraft:air" || location_2_2.typeId == "minecraft:water" || location_2_2.typeId == "minecraft:flowing_water") && (location_2_3.typeId == "minecraft:air" || location_2_3.typeId == "minecraft:water" || location_2_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_2_5.typeId == "minecraft:air" || location_2_5.typeId == "minecraft:water" || location_2_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_2_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_2_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_2_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_2_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "south")  {
                if((location_3_1.typeId == "minecraft:air" || location_3_1.typeId == "minecraft:water" || location_3_1.typeId == "minecraft:flowing_water") && (location_3_2.typeId == "minecraft:air" || location_3_2.typeId == "minecraft:water" || location_3_2.typeId == "minecraft:flowing_water") && (location_3_3.typeId == "minecraft:air" || location_3_3.typeId == "minecraft:water" || location_3_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_3_5.typeId == "minecraft:air" || location_3_5.typeId == "minecraft:water" || location_3_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_3_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_3_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_3_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_3_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "west")  {
                if((location_4_1.typeId == "minecraft:air" || location_4_1.typeId == "minecraft:water" || location_4_1.typeId == "minecraft:flowing_water") && (location_4_2.typeId == "minecraft:air" || location_4_2.typeId == "minecraft:water" || location_4_2.typeId == "minecraft:flowing_water") && (location_4_3.typeId == "minecraft:air" || location_4_3.typeId == "minecraft:water" || location_4_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_4_5.typeId == "minecraft:air" || location_4_5.typeId == "minecraft:water" || location_4_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_4_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_4_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_4_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_4_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_17_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s3_p1_18_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            
            const location_1_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1});
            const location_1_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z - 1 });
            const location_1_3 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z });
            const location_1_5 = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z - 1 });

            const location_2_1 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z});
            const location_2_2 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z - 1 });
            const location_2_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1 });
            const location_2_5 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y + 1, z: block.location.z });

            const location_3_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1});
            const location_3_2 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z + 1 });
            const location_3_3 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z });
            const location_3_5 = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z + 1 });

            const location_4_1 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z});
            const location_4_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z + 1 });
            const location_4_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1 });
            const location_4_5 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y + 1, z: block.location.z });
            
            const location_1234_4 = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });

            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];

            if(playerDirection == "north") {
                if((location_1_1.typeId == "minecraft:air" || location_1_1.typeId == "minecraft:water" || location_1_1.typeId == "minecraft:flowing_water") && (location_1_2.typeId == "minecraft:air" || location_1_2.typeId == "minecraft:water" || location_1_2.typeId == "minecraft:flowing_water") && (location_1_3.typeId == "minecraft:air" || location_1_3.typeId == "minecraft:water" || location_1_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_1_5.typeId == "minecraft:air" || location_1_5.typeId == "minecraft:water" || location_1_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_1_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_1_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_1_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_1_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "east")  {
                if((location_2_1.typeId == "minecraft:air" || location_2_1.typeId == "minecraft:water" || location_2_1.typeId == "minecraft:flowing_water") && (location_2_2.typeId == "minecraft:air" || location_2_2.typeId == "minecraft:water" || location_2_2.typeId == "minecraft:flowing_water") && (location_2_3.typeId == "minecraft:air" || location_2_3.typeId == "minecraft:water" || location_2_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_2_5.typeId == "minecraft:air" || location_2_5.typeId == "minecraft:water" || location_2_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_2_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_2_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_2_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_2_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "south")  {
                if((location_3_1.typeId == "minecraft:air" || location_3_1.typeId == "minecraft:water" || location_3_1.typeId == "minecraft:flowing_water") && (location_3_2.typeId == "minecraft:air" || location_3_2.typeId == "minecraft:water" || location_3_2.typeId == "minecraft:flowing_water") && (location_3_3.typeId == "minecraft:air" || location_3_3.typeId == "minecraft:water" || location_3_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_3_5.typeId == "minecraft:air" || location_3_5.typeId == "minecraft:water" || location_3_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_3_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_3_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_3_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_3_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "west")  {
                if((location_4_1.typeId == "minecraft:air" || location_4_1.typeId == "minecraft:water" || location_4_1.typeId == "minecraft:flowing_water") && (location_4_2.typeId == "minecraft:air" || location_4_2.typeId == "minecraft:water" || location_4_2.typeId == "minecraft:flowing_water") && (location_4_3.typeId == "minecraft:air" || location_4_3.typeId == "minecraft:water" || location_4_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_4_5.typeId == "minecraft:air" || location_4_5.typeId == "minecraft:water" || location_4_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_4_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_4_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_4_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_4_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_18_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            }
        }
    });
    data.blockComponentRegistry.registerCustomComponent("rzb_dcocrfnt:place_s3_p1_19_parts", {
        beforeOnPlayerPlace: (eventData) => {
            const block = eventData.block;
            
            const location_1_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1});
            const location_1_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z - 1 });
            const location_1_3 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z });
            const location_1_5 = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z - 1 });

            const location_2_1 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z});
            const location_2_2 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z - 1 });
            const location_2_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z - 1 });
            const location_2_5 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y + 1, z: block.location.z });

            const location_3_1 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1});
            const location_3_2 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z + 1 });
            const location_3_3 = block.dimension.getBlock({ x: block.location.x + 1, y: block.location.y, z: block.location.z });
            const location_3_5 = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z + 1 });

            const location_4_1 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z});
            const location_4_2 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y, z: block.location.z + 1 });
            const location_4_3 = block.dimension.getBlock({ x: block.location.x, y: block.location.y, z: block.location.z + 1 });
            const location_4_5 = block.dimension.getBlock({ x: block.location.x - 1, y: block.location.y + 1, z: block.location.z });
            
            const location_1234_4 = block.dimension.getBlock({ x: block.location.x, y: block.location.y + 1, z: block.location.z });

            const player = eventData.player;
        
            const { y: yaw } = player.getRotation();
            const normalizedYaw = (yaw % 360 + 360) % 360;
            const index = Math.floor((normalizedYaw + 45) / 90) % 4;
            const playerDirection = directions[index];
            const playerDirectionTwo = directionsTwo[index];

            if(playerDirection == "north") {
                if((location_1_1.typeId == "minecraft:air" || location_1_1.typeId == "minecraft:water" || location_1_1.typeId == "minecraft:flowing_water") && (location_1_2.typeId == "minecraft:air" || location_1_2.typeId == "minecraft:water" || location_1_2.typeId == "minecraft:flowing_water") && (location_1_3.typeId == "minecraft:air" || location_1_3.typeId == "minecraft:water" || location_1_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_1_5.typeId == "minecraft:air" || location_1_5.typeId == "minecraft:water" || location_1_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_1_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_1_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_1_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_1_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "east")  {
                if((location_2_1.typeId == "minecraft:air" || location_2_1.typeId == "minecraft:water" || location_2_1.typeId == "minecraft:flowing_water") && (location_2_2.typeId == "minecraft:air" || location_2_2.typeId == "minecraft:water" || location_2_2.typeId == "minecraft:flowing_water") && (location_2_3.typeId == "minecraft:air" || location_2_3.typeId == "minecraft:water" || location_2_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_2_5.typeId == "minecraft:air" || location_2_5.typeId == "minecraft:water" || location_2_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_2_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_2_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_2_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_2_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "south")  {
                if((location_3_1.typeId == "minecraft:air" || location_3_1.typeId == "minecraft:water" || location_3_1.typeId == "minecraft:flowing_water") && (location_3_2.typeId == "minecraft:air" || location_3_2.typeId == "minecraft:water" || location_3_2.typeId == "minecraft:flowing_water") && (location_3_3.typeId == "minecraft:air" || location_3_3.typeId == "minecraft:water" || location_3_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_3_5.typeId == "minecraft:air" || location_3_5.typeId == "minecraft:water" || location_3_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_3_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_3_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_3_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_3_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            } else if(playerDirection == "west")  {
                if((location_4_1.typeId == "minecraft:air" || location_4_1.typeId == "minecraft:water" || location_4_1.typeId == "minecraft:flowing_water") && (location_4_2.typeId == "minecraft:air" || location_4_2.typeId == "minecraft:water" || location_4_2.typeId == "minecraft:flowing_water") && (location_4_3.typeId == "minecraft:air" || location_4_3.typeId == "minecraft:water" || location_4_3.typeId == "minecraft:flowing_water") && (location_1234_4.typeId == "minecraft:air" || location_1234_4.typeId == "minecraft:water" || location_1234_4.typeId == "minecraft:flowing_water") && (location_4_5.typeId == "minecraft:air" || location_4_5.typeId == "minecraft:water" || location_4_5.typeId == "minecraft:flowing_water")) {
                    block.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirection, "rzb_dcocrfnt:part": 0 }));
                    location_4_1.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 1 }));
                    location_4_2.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 2 }));
                    location_4_3.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 3 }));
                    location_1234_4.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 4 }));
                    location_4_5.setPermutation(BlockPermutation.resolve('rzb_dcocrfnt:deco_s3_p1_19_parts', { "minecraft:cardinal_direction": playerDirectionTwo, "rzb_dcocrfnt:part": 5 }));
                } else {
                    eventData.cancel = true;
                    player.sendMessage({ translate : "dialogue.decocraft_message.place_deco" });
                    player.playSound("note.didgeridoo", soundOptions3);
                }
            }
        }
    });
});